package com.dxhy.common.enums;

/**
 * @author jiaohongyang
 */
public enum FplxEnum {
    /**
     * 发票类型
     * 10:增值税专用发票
     * 11:增值税普通发票
     * 12:增值税电子普通发票
     * 13:增值税普通发票(卷票)
     * 14:机动车销售统一发票
     * 15:二手车销售统一发票
     * 16:定额发票
     * 17:机打发票
     * 18:出租车发票
     * 19:火车票
     * 20:客运汽车
     * 21:航空运输电子客票行程单
     * 22:过路费发票
     * 24:增值税电子普通发票(通行费)
     * 25:增值税电子专用发票
     * 26:电子发票《增值税专用发票)
     * 27:电子发票《普通发票)
     */

    /**
     * 专票代码
     */
    ZP("01", 1,"10"),
    /**
     * 机动车代码
     */
    JDC("03", 2,"14"),
    /**
     * 普票代码
     */
    PP("04", 3,"11"),
    /**
     * 电子票代码
     */
    DZP("10", 4,"12"),
    /**
     * 卷式发票代码
     */
    JSP("11", 5,"13"),
    /**
     * 通行费发票代码
     */
    TXF("14", 6,"24"),

    ESC("15", 7,"15"),
    /**
     * 电子专票
     */
    DZZP("08", 8,"25"),
    /**
     * 海关缴款书
     */
    HGJKS("17", 9,""),
    /**
     * 电子发票（增值税专用发票）
     */
    QDZZP("31", 10,"26"),
    /**
     * 电子发票（普通发票）
     */
    QDPP("32", 11,"27"),
    /**
     * 机动车销售电子统一发票
     */
    JDCDZ("183", 12,"14"),
    /**
     * 二手车销售电子统一发票
     */
    ESCDZ("184", 13,"15"),
    /**
     * 纸质发票(增值税专用发票)
     */
    ZZDZ("185", 14,"10"),
    /**
     * 纸质发票(普通发票)
     */
    ZZPP("186", 15,"11"),
    /**
     * 纸质发票（机动车销售统一发票）
     */
    ZZJDC("187", 16,"14"),
    /**
     * 纸质发票（二手车销售统一发票）
     */
    ZZESC("188", 17,"15"),
    /**
     * 增加电子发票（铁路电子客票）
     */
    TLKP("283", 18,"19"),
    /**
     * 电子发票（航空运输客票电子行程单）
     */
    HKDZFP("161", 19,"21"),
    /**
     * 货物运输业增值税专用发票
     */
    HWYH("02", 20,"");

    private final String[] FPLXMC = {"增值税专用发票", "机动车销售统一发票", "增值税普通发票", "增值税电子普通发票", "增值税普通发票（卷票）", "增值税电子普通发票（通行费）",
            "二手车销售统一发票", "增值税电子专用发票", "海关缴款书", "电子发票（增值税专用发票）", "电子发票（普通发票）", "机动车销售电子统一发票", "二手车销售电子统一发票",
            "纸质发票(增值税专用发票)", "纸质发票(普通发票)", "纸质发票（机动车销售统一发票）", "纸质发票（二手车销售统一发票）", "增加电子发票（铁路电子客票）",
            "电子发票（航空运输客票电子行程单）", "货物运输业增值税专用发票"};

    private String fplxDm;
    private int index;

    private String sapFplxdm;

    FplxEnum(String fplxDm, int index,String sapFplxdm) {
        this.fplxDm = fplxDm;
        this.index = index;
        this.sapFplxdm = sapFplxdm;
    }

    public String getFplxDm(int index) {
        for (FplxEnum fplx : FplxEnum.values()) {
            if (fplx.getIndex() == index) {
                return fplx.fplxDm;
            }
        }
        return null;
    }

    public static String getSapFplxdm(String fplxdm) {
        for (FplxEnum fplx : FplxEnum.values()) {
            if (fplx.getFplxDm().equals(fplxdm)) {
                return fplx.sapFplxdm;
            }
        }
        return null;
    }

    public String getFplxMc(int index) {
        int length = FPLXMC.length;
        if (index <= length) {
            return FPLXMC[index - 1];
        }
        return null;
    }

    public String getFplxDm() {
        return fplxDm;
    }

    public void setFplxDm(String fplxDm) {
        this.fplxDm = fplxDm;
    }

    public int getIndex() {
        return index;
    }

    public void setIndex(int index) {
        this.index = index;
    }

    public String getSapFplxdm() {
        return sapFplxdm;
    }

    public void setSapFplxdm(String sapFplxdm) {
        this.sapFplxdm = sapFplxdm;
    }
}
