package com.dxhy.core.thread;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dxhy.common.constant.DbConstant;
import com.dxhy.common.constant.DictConstant;
import com.dxhy.common.datasource.config.DynamicContextHolder;
import com.dxhy.common.service.DictdetaServcice;
import com.dxhy.common.service.impl.TabulateRedisServiceImpl;
import com.dxhy.common.util.ExcelPoiUtil;
import com.dxhy.common.util.ExceptionUtil;
import com.dxhy.common.util.MathUtil;
import com.dxhy.common.util.StringHelper;
import com.dxhy.common.utils.DateUtils;
import com.dxhy.core.entity.TDxExcelExportlog;
import com.dxhy.core.entity.TDxRecordInvoice;
import com.dxhy.core.feign.IFeginWebsocket;
import com.dxhy.core.model.CautionMaximalReqVO;
import com.dxhy.core.model.ExceptionInvoiceVo;
import com.dxhy.core.rabbitmq.Sender;
import com.dxhy.core.service.export.IExportCautionService;
import com.dxhy.core.service.singletable.IExcelExportlogService;
import com.dxhy.core.util.InvoiceTypeUtils;
import com.dxhy.core.util.JsonResponseUtils;
import com.dxhy.oss.service.FileService;
import com.github.pagehelper.PageInfo;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author user
 * @date 2019-05-20 逾期预警导出
 */

@Slf4j

public class CautionExportMaximalThread extends BaseThread {

    /**
     * 默认要导出第几页的数据
     */
    private final int default_curr = 1;
    /**
     * 默认每页导出多少条数据
     */
    private final int default_size = 5000;
    /**
     * 文件名前缀
     */
    private final String fileNamePrefix = "cautionMaximal";
    /**
     * 文件名后缀
     */
    private final String fileNameSurfix = ".xls";
    /**
     * 文件名中间分隔符
     */
    private final String fileSplit = "_";
    /**
     * excel的标题
     */
    private final String title = "普票跨年度预警";
    /**
     * 导出excel的列名
     */
    private final String[] titleColumns = new String[] {"序号", "签收日期", "签收状态", "签收人", "发票代码", "发票号码", "发票类型", "开票日期",
        "购方名称", "购方税号", "销方名称", "销方税号", "金额", "税额"};

    private final List<TDxExcelExportlog> exportLogs;

    private final IExcelExportlogService exportLogService;

    private final IExportCautionService cautionService;

    private final FileService fileService;

    private final Sender sender;
    private final TabulateRedisServiceImpl tabulateRedisService;
    private final DictdetaServcice dictdetaServcice;

    public CautionExportMaximalThread(IExcelExportlogService exportLogService, List<TDxExcelExportlog> exportLogs,
        IExportCautionService cautionService, FileService fileService, Sender sender, IFeginWebsocket feginWebsocket,
        TabulateRedisServiceImpl tabulateRedisService, DictdetaServcice dictdetaServcice) {
        this.exportLogService = exportLogService;
        this.exportLogs = exportLogs;
        this.cautionService = cautionService;
        this.fileService = fileService;
        this.sender = sender;
        this.feginWebsocket = feginWebsocket;
        this.tabulateRedisService = tabulateRedisService;
        this.dictdetaServcice = dictdetaServcice;
    }

    private final IFeginWebsocket feginWebsocket;

    @Override
    public void run() {
        for (TDxExcelExportlog exportLog : exportLogs) {
            // 日志表主键
            Long id = exportLog.getId();
            String userid = exportLog.getUserAccount();
            String userName = exportLog.getUserName();

            // 导出条件
            String condition = exportLog.getConditions();

            // 数据源名称
            String dbName = exportLog.getDbName();
            if (StringHelper.isBlank(dbName)) {
                dbName = "";
            }
            String createDate = DateUtils.dateToStrLong(exportLog.getCreateDate());

            JSONObject msg = new JSONObject();

            // 捕获异常，防止某一条日志导出出错，影响其它日志的导出
            try {
                exportLogService.updateStart(id, dbName);

                // 应该保存的参数
                JSONObject params = JSONObject.parseObject(condition);

                CautionMaximalReqVO reqVO = JSONObject.parseObject(params.toJSONString(), CautionMaximalReqVO.class);
                JSONObject jsonObject = tabulateRedisService.getTabulateExcel(reqVO.getCompany(), reqVO.getMenuId());
                // 导出文件名
                StringBuilder ftpFileName = new StringBuilder();
                ftpFileName.append(userid);
                ftpFileName.append(fileSplit);
                ftpFileName.append(fileNamePrefix);
                ftpFileName.append(fileSplit);
                ftpFileName.append(DateUtils.getStringDateShort());
                ftpFileName.append(fileSplit);
                ftpFileName.append((new Date()).getTime());
                ftpFileName.append(fileNameSurfix);

                // 本地文件存储路径
                StringBuilder excelFile = new StringBuilder();
                excelFile.append(fileService.getLocalPathDefault());
                excelFile.append(ftpFileName.toString());

                // 递归导出数据到excel
                this.cicleExport(reqVO, default_curr, excelFile.toString(), exportLog, jsonObject);

                // FTP文件存储路径
                String[] toDay = DateUtil.today().split("-");
                String ftpFilePath = fileService.getPathPrefix() + userName + "/" + "excelFile/" + toDay[0] + "/"
                    + toDay[1] + "/" + toDay[2] + "/" + ftpFileName.toString();
                // 文件上传ftp
                fileService.uploadFile(ftpFilePath, excelFile.toString());

                // 设置成功消息内容
                msg.put("title", super.getSuccTitle(title));
                msg.put("content", super.getSuccContent(createDate));
                msg.put("url", super.getUrl(id));
                msg.put("useraccount", userName);

                // 记录导出完成状态
                exportLogService.updateSucc(id, ftpFilePath, dbName);

            } catch (Exception e) {
                String errmsg = ExceptionUtil.getExceptionDeteil(e, 4000);
                // 设置失败消息内容
                msg.put("title", super.getFailTitle(title));
                msg.put("content", super.getFailContent(createDate, errmsg));
                msg.put("url", "");
                msg.put("useraccount", userName);

                log.error("", e);
                // 记录导出出错状态
                exportLogService.updateFail(id, errmsg, dbName);
            } finally {
                // 发送导出消息到mq
                String responseResult = JsonResponseUtils.responseResult(msg);
                sender.sendToMsg(responseResult);
                // 申请成功后像websocket发送申请信息
                feginWebsocket.setMessageToWebsocket(userName);
            }
        }
    }

    /**
     * @param reqVO
     * @param defaultCurr
     * @param excelFile
     * @param exportLog
     * @throws Exception
     */
    private void cicleExport(CautionMaximalReqVO reqVO, int defaultCurr, String excelFile, TDxExcelExportlog exportLog,
        JSONObject jsonObject) throws Exception {
        List<String> allTaxNo = new ArrayList<>();
        if (!StringHelper.isBlank(reqVO.getGfsh())) {
            allTaxNo = StringHelper.stringToList(reqVO.getGfsh());
        }
        reqVO.setAllTaxNo(allTaxNo);
        Page<TDxRecordInvoice> page = new Page<>(defaultCurr, default_size);
        PageInfo<TDxRecordInvoice> pageList = cautionService.listMaximal(page, reqVO, allTaxNo);
        List<ExceptionInvoiceVo> dataList = convertToMaximalData(pageList.getList());

        boolean firstPage = false;
        if (pageList.isIsFirstPage()) {
            firstPage = true;
        }

        // 已经是最后一页
        if (!pageList.isIsLastPage()) {
            // 写入数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, false);
        } else if (pageList.isIsLastPage()) {
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, true);
            return;
        }
        // 更新页数
        defaultCurr++;
        cicleExport(reqVO, defaultCurr, excelFile, exportLog, jsonObject);
    }

    /**
     * 普票跨年度预警view
     *
     * @param list
     * @return
     */
    private List<ExceptionInvoiceVo> convertToMaximalData(List<TDxRecordInvoice> list) {
        List<ExceptionInvoiceVo> dataList = new ArrayList<>();
        int xh = 1;
        for (TDxRecordInvoice record : list) {
            ExceptionInvoiceVo vo = new ExceptionInvoiceVo();
            vo.setXh(xh + "");
            xh++;
            // 签收日期
            vo.setQsDate(DateUtils.dateToStr(record.getQsDate()));
            // 签收结果
            vo.setQsStatusName(dictdetaServcice.queryDictName(DictConstant.QS_RESULTS, record.getQsStatus()));
            // 发票代码
            vo.setInvoiceCode(record.getInvoiceCode());
            // 发票号码
            vo.setInvoiceNo(record.getInvoiceNo());
            // 发票类型
            vo.setInvoiceType(new InvoiceTypeUtils().invoiceTypeName(record.getInvoiceType()));
            // 开票日期
            vo.setInvoiceDate(DateUtils.dateToStr(record.getInvoiceDate()));
            // 购方名称
            vo.setBuyerName(record.getGfName());
            vo.setSellerName(record.getXfName());
            // 未税金额
            vo.setNoTaxAmount(MathUtil.round(record.getInvoiceAmount().toPlainString()));
            // 税额
            vo.setTaxAmount(MathUtil.round(record.getTaxAmount().toPlainString()));

            dataList.add(vo);
        }
        return dataList;
    }
}
