package com.dxhy.core.thread;

import java.util.*;

import com.alibaba.fastjson.JSONObject;
import com.dxhy.common.service.impl.TabulateRedisServiceImpl;
import com.dxhy.common.util.ExcelPoiUtil;
import com.dxhy.common.util.ExceptionUtil;
import com.dxhy.common.util.StringHelper;
import com.dxhy.common.utils.DateUtils;
import com.dxhy.common.vo.Business;
import com.dxhy.core.entity.TDxExcelExportlog;
import com.dxhy.core.feign.IFeginWebsocket;
import com.dxhy.core.model.JkszhcxRespVO;
import com.dxhy.core.rabbitmq.Sender;
import com.dxhy.core.service.export.JkszhcxService;
import com.dxhy.core.service.singletable.IExcelExportlogService;
import com.dxhy.core.util.JsonResponseUtils;
import com.dxhy.oss.service.FileService;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 海关缴款书综合查询导出
 *
 * @author aa
 */
@SuppressWarnings({"AlibabaMethodTooLong", "DuplicatedCode", "FieldCanBeLocal"})
@Slf4j

public class JkszhcxExportThread extends BaseThread {

    /**
     * 默认要导出第几页的数据
     */
    private final int default_curr = 1;
    /**
     * 默认每页导出多少条数据
     */
    private final int default_size = 5000;
    /**
     * 文件名前缀
     */
    private final String fileNamePrefix = "jkszhcxList";
    /**
     * 文件名后缀
     */
    private final String fileNameSurfix = ".xls";
    /**
     * 文件名中间分隔符
     */
    private final String fileSplit = "_";
    /**
     * excel的标题
     */
    private final String title = "海关缴款书综合查询";
    /**
     * 导出excel的列名
     */
    private final String[] titleColumns =
        {"序号", "缴款书号码", "填发日期", "缴款单位税号", "缴款单位名称", "税款金额", "抵扣税款金额", "稽核结果", "是否申请核对", "核查结果", "签收状态", "签收日期", "签收方式",
            "签收人", "认证时间", "认证状态", "认证处理状态", "认证类型", "认证人", "税款所属期", "管理状态", "业务单元"};

    private final IExcelExportlogService exportLogService;

    private final List<TDxExcelExportlog> exportLogs;

    private final JkszhcxService jkszhcxService;

    private final FileService fileService;

    private final Sender sender;
    private final TabulateRedisServiceImpl tabulateRedisService;

    public JkszhcxExportThread(IExcelExportlogService exportLogService, List<TDxExcelExportlog> exportLogs,
        JkszhcxService jkszhcxService, FileService fileService, Sender sender, IFeginWebsocket feginWebsocket,
        TabulateRedisServiceImpl tabulateRedisService) {
        this.exportLogService = exportLogService;
        this.exportLogs = exportLogs;
        this.jkszhcxService = jkszhcxService;
        this.fileService = fileService;
        this.sender = sender;
        this.feginWebsocket = feginWebsocket;
        this.tabulateRedisService = tabulateRedisService;
    }

    private final IFeginWebsocket feginWebsocket;

    @Override
    public void run() {
        for (TDxExcelExportlog exportLog : exportLogs) {
            // 日志表主键
            Long id = exportLog.getId();
            String userid = exportLog.getUserAccount();

            String userName = exportLog.getUserName();
            // 导出条件

            String condition = exportLog.getConditions();
            // 数据源名称
            String dbName = exportLog.getDbName();
            if (StringHelper.isBlank(dbName)) {
                dbName = "";
            }
            String createDate = DateUtils.dateToStrLong(exportLog.getCreateDate());

            JSONObject msg = new JSONObject();

            // 捕获异常，防止某一条日志导出出错，影响其它日志的导出
            try {
                exportLogService.updateStart(id, dbName);

                // 应该保存的参数
                JSONObject prams = JSONObject.parseObject(condition);
                String gfsh = prams.getString("gfsh");
                String companyCode = prams.getString("companyCode");
                String customsCode = prams.getString("customsCode");
                String fillInStartDate = prams.getString("fillInStartDate");
                String fillInEndDate = prams.getString("fillInEndDate");
                String taxAmount = prams.getString("taxAmount");
                String qsStatus = prams.getString("qsStatus");
                String rzhYesorno = prams.getString("rzhYesorno");
                String bdkStatus = prams.getString("bdkStatus");
                String confirmUser = prams.getString("confirmUser");
                String glzt = prams.getString("glzt");
                String auditStatus = prams.getString("auditStatus");
                String applyCheck = prams.getString("applyCheck");
                String checkStatus = prams.getString("checkStatus");
                String qsDateStartDate = prams.getString("qsDateStartDate");
                String qsDateEndDate = prams.getString("qsDateEndDate");
                String rzhStartDate = prams.getString("rzhStartDate");
                String rzhEndDate = prams.getString("rzhEndDate");
                String rzhBelongDate = prams.getString("rzhBelongDate");
                List<Business> businesses = (List<Business>)prams.get("userBusiness");
                String businessCode = prams.getString("businessCode");
                String company = prams.getString("company");
                String menuId = prams.getString("menuId");
                JSONObject jsonObject = tabulateRedisService.getTabulateExcel(company, menuId);
                // 参数转换为需要的形式
                Map<String, Object> pramsMap = new HashMap<>(16);
                pramsMap.put("dbName", dbName);
                pramsMap.put("business", businesses);

                pramsMap.put("gfsh", gfsh);
                pramsMap.put("companyCode", companyCode);
                if (!StringHelper.isBlank(customsCode)) {
                    pramsMap.put("customsCode", customsCode);
                }
                if (!StringHelper.isBlank(fillInStartDate)) {
                    pramsMap.put("fillInStartDate", DateUtils.getMorning(DateUtils.strToDate(fillInStartDate)));
                }
                if (!StringHelper.isBlank(fillInEndDate)) {
                    pramsMap.put("fillInEndDate", DateUtils.getNight(DateUtils.strToDate(fillInEndDate)));
                }
                if (!StringHelper.isBlank(taxAmount)) {
                    pramsMap.put("taxAmount", taxAmount);
                }
                if (!StringHelper.isBlank(qsStatus)) {
                    if (!"99".equals(qsStatus)) {
                        pramsMap.put("qsStatus", qsStatus);
                        if ("1".equals(qsStatus)) {
                            if (!StringHelper.isBlank(qsDateStartDate)) {
                                pramsMap.put("qsDateStartDate",
                                    DateUtils.getMorning(DateUtils.strToDate(qsDateStartDate)));
                            }
                            if (!StringHelper.isBlank(qsDateEndDate)) {
                                pramsMap.put("qsDateEndDate", DateUtils.getNight(DateUtils.strToDate(qsDateEndDate)));
                            }
                            if (!"99".equals(businessCode)) {
                                pramsMap.put("businessCode", businessCode);
                            }
                        }
                    }
                }
                if (!StringHelper.isBlank(rzhYesorno)) {
                    if (!"99".equals(rzhYesorno)) {
                        pramsMap.put("rzhYesorno", rzhYesorno);
                        if (!"0".equals(rzhYesorno)) {
                            if (!StringHelper.isBlank(rzhStartDate)) {
                                pramsMap.put("rzhStartDate", DateUtils.getMorning(DateUtils.strToDate(rzhStartDate)));
                            }
                            if (!StringHelper.isBlank(rzhEndDate)) {
                                pramsMap.put("rzhEndDate", DateUtils.getNight(DateUtils.strToDate(rzhEndDate)));
                            }
                            if (!StringHelper.isBlank(rzhBelongDate)) {
                                pramsMap.put("rzhBelongDate", rzhBelongDate);
                            }
                        }
                    }
                }
                if (!StringHelper.isBlank(bdkStatus)) {
                    if (!"99".equals(bdkStatus)) {
                        pramsMap.put("bdkStatus", bdkStatus);
                    }
                }
                if (!StringHelper.isBlank(confirmUser)) {
                    pramsMap.put("confirmUser", confirmUser);
                }
                if (!StringHelper.isBlank(glzt)) {
                    if (!"99".equals(glzt)) {
                        pramsMap.put("glzt", glzt);
                    }
                }
                if (!StringHelper.isBlank(auditStatus)) {
                    if (!"99".equals(auditStatus)) {
                        pramsMap.put("auditStatus", auditStatus);
                    }
                }
                if (!StringHelper.isBlank(applyCheck)) {
                    if (!"99".equals(applyCheck)) {
                        pramsMap.put("applyCheck", applyCheck);
                    }
                }
                if (!StringHelper.isBlank(checkStatus)) {
                    if (!"99".equals(checkStatus)) {
                        pramsMap.put("checkStatus", checkStatus);
                    }
                }
                if (!StringHelper.isBlank(auditStatus)) {
                    if (!"99".equals(auditStatus)) {
                        pramsMap.put("auditStatus", auditStatus);
                    }
                }
                pramsMap.put("company", company);
                // 导出文件名
                StringBuilder ftpFileName = new StringBuilder();
                ftpFileName.append(userid);
                ftpFileName.append(fileSplit);
                ftpFileName.append(fileNamePrefix);
                ftpFileName.append(fileSplit);
                ftpFileName.append(DateUtils.getStringDateShort());
                ftpFileName.append(fileSplit);
                ftpFileName.append((new Date()).getTime());
                ftpFileName.append(fileNameSurfix);

                // 本地文件存储路径
                StringBuilder excelFile = new StringBuilder();
                excelFile.append(fileService.getLocalPathDefault());
                excelFile.append(ftpFileName.toString());

                // 递归导出数据到excel
                this.cicleExport(pramsMap, default_curr, excelFile.toString(), jsonObject);

                // FTP文件存储路径
                String[] toDay = DateUtil.today().split("-");
                String ftpFilePath = fileService.getPathPrefix() + userName + "/" + "excelFile/" + toDay[0] + "/"
                    + toDay[1] + "/" + toDay[2] + "/" + ftpFileName.toString();
                // 文件上传ftp
                fileService.uploadFile(ftpFilePath, excelFile.toString());

                // 设置成功消息内容
                msg.put("title", super.getSuccTitle(title));
                msg.put("content", super.getSuccContent(createDate));
                msg.put("url", super.getUrl(id));
                msg.put("useraccount", userName);

                // 记录导出完成状态
                exportLogService.updateSucc(id, ftpFilePath, dbName);

            } catch (Exception e) {
                String errmsg = ExceptionUtil.getExceptionDeteil(e, 4000);
                // 设置失败消息内容
                msg.put("title", super.getFailTitle(title));
                msg.put("content", super.getFailContent(createDate, errmsg));
                msg.put("url", "");
                msg.put("useraccount", userName);

                log.error("", e);
                // 记录导出出错状态
                exportLogService.updateFail(id, errmsg, dbName);
            } finally {
                // 发送导出消息到mq
                String responseResult = JsonResponseUtils.responseResult(msg);
                sender.sendToMsg(responseResult);

                // 申请成功后像websocket发送申请信息
                feginWebsocket.setMessageToWebsocket(userName);
            }
        }
    }

    /**
     * 递归导出数据到excel文件，防止jvm oom
     *
     * @param pramsMap
     *            查询数据参数集合
     * @param curr
     *            当前第几页
     * @param excelFile
     *            导出的文件
     * @throws Exception
     *             异常
     *
     */

    private boolean cicleExport(Map<String, Object> pramsMap, int curr, String excelFile, JSONObject jsonObject)
        throws Exception {
        // 查询指定页数的数据
        JSONObject pageData = jkszhcxService.selectByPramsMap(pramsMap, curr, default_size);
        boolean hasNext = pageData.getBooleanValue("hasNext");
        List<JkszhcxRespVO> dataList = (List<JkszhcxRespVO>)pageData.get("datalist");
        boolean firstPage = false;
        if (curr == default_curr) {
            firstPage = true;
        }

        // 数据不是最后一页
        if (hasNext) {
            int nextPage = pageData.getIntValue("nextPage");
            // 写入当前页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, false);
            // 更新页数
            return cicleExport(pramsMap, nextPage, excelFile, jsonObject);
        } else {
            // 写入最后一页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, true);
            return false;
        }

    }
}
