package com.dxhy.core.thread;

import java.util.*;

import com.alibaba.fastjson.JSONObject;
import com.dxhy.common.service.impl.TabulateRedisServiceImpl;
import com.dxhy.common.util.ExcelPoiUtil;
import com.dxhy.common.util.ExceptionUtil;
import com.dxhy.common.util.StringHelper;
import com.dxhy.common.utils.DateUtils;
import com.dxhy.common.vo.Business;
import com.dxhy.core.entity.TDxExcelExportlog;
import com.dxhy.core.feign.IFeginWebsocket;
import com.dxhy.core.model.BbycfpcxRespVO;
import com.dxhy.core.rabbitmq.Sender;
import com.dxhy.core.service.export.YcfpcxService;
import com.dxhy.core.service.singletable.IExcelExportlogService;
import com.dxhy.core.util.JsonResponseUtils;
import com.dxhy.oss.service.FileService;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

/**
 * 报表发票综合查询导出
 *
 * @author aa
 */

@SuppressWarnings("AlibabaMethodTooLong")
@Slf4j

public class YcfpcxExportThread extends BaseThread {

    /**
     * 默认要导出第几页的数据
     */
    private final int default_curr = 1;
    /**
     * 默认每页导出多少条数据
     */
    private final int default_size = 5000;
    /**
     * 文件名前缀
     */
    private final String fileNamePrefix = "bbfpzhcxList";
    /**
     * 文件名后缀
     */
    private final String fileNameSurfix = ".xls";
    /**
     * 文件名中间分隔符
     */
    private final String fileSplit = "_";
    /**
     * excel的标题
     */
    private final String title = "异常发票查询";
    /**
     * 导出excel的列名
     */
    private final String[] titleColumns = {"序号", "发票代码", "发票号码", "发票类型", "开票日期", "购方税号", "购方名称", "销方税号", "销方名称", "金额",
        "税额", "异常状态", "签收日期", "异常时间", "签收状态", "签收人", "认证时间", "认证状态", "认证处理状态", "认证类型", "税款所属期", "业务单元"};

    private final IExcelExportlogService exportLogService;

    private final List<TDxExcelExportlog> exportLogs;

    private final YcfpcxService ycfpcxService;

    private final FileService fileService;

    private final Sender sender;
    private final TabulateRedisServiceImpl tabulateRedisService;

    public YcfpcxExportThread(IExcelExportlogService exportLogService, List<TDxExcelExportlog> exportLogs,
        YcfpcxService ycfpcxService, FileService fileService, Sender sender, IFeginWebsocket feginWebsocket,
        TabulateRedisServiceImpl tabulateRedisService) {

        this.exportLogService = exportLogService;
        this.exportLogs = exportLogs;
        this.ycfpcxService = ycfpcxService;
        this.fileService = fileService;
        this.sender = sender;
        this.feginWebsocket = feginWebsocket;
        this.tabulateRedisService = tabulateRedisService;
    }

    private final IFeginWebsocket feginWebsocket;

    @Override
    public void run() {
        for (TDxExcelExportlog exportLog : exportLogs) {
            // 日志表主键
            Long id = exportLog.getId();
            String userid = exportLog.getUserAccount();

            String userName = exportLog.getUserName();
            // 导出条件
            String condition = exportLog.getConditions();
            // 数据源名称
            String dbName = exportLog.getDbName();
            if (StringHelper.isBlank(dbName)) {
                dbName = "";
            }
            String createDate = DateUtils.dateToStrLong(exportLog.getCreateDate());

            JSONObject msg = new JSONObject();

            // 捕获异常，防止某一条日志导出出错，影响其它日志的导出
            try {
                exportLogService.updateStart(id, dbName);

                // 应该保存的参数
                JSONObject prams = JSONObject.parseObject(condition);
                String gfsh = prams.getString("gfsh");
                String companyCode = prams.getString("companyCode");
                String kpksrq = prams.getString("kpksrq");
                String kpjsrq = prams.getString("kpjsrq");
                String qszt = prams.getString("qszt");
                String rzzt = prams.getString("rzzt");
                String yczt = prams.getString("yczt");
                String invoiceNo = prams.getString("invoiceNo");
                String xfmc = prams.getString("xfmc");
                String company = prams.getString("company");
                String menuId = prams.getString("menuId");
                List<Business> businessList = (List<Business>)prams.get("business");
                String businessCode = prams.getString("businessCode");
                String bzr = prams.getString("bzr");
                String bzdh = prams.getString("bzdh");
                String imageNumber = prams.getString("imageNumber");
                String inAccountStatus = prams.getString("inAccountStatus");
                String snVoucherNumber =prams.getString("snVoucherNumber");
                String accountPeriod = prams.getString("accountPeriod");
                if (StringUtils.isNotEmpty(accountPeriod) && accountPeriod.length() > 7){
                    accountPeriod = accountPeriod.substring(0,7);
                }
                String postingTimeEnd = prams.getString("postingTimeEnd");
                String postingTimeStart = prams.getString("postingTimeStart");
                JSONObject jsonObject = tabulateRedisService.getTabulateExcel(company, menuId);
                // 参数转换为需要的形式
                Map<String, Object> pramsMap = new HashMap<>(16);
                pramsMap.put("dbName", dbName);
                pramsMap.put("business", businessList);
                pramsMap.put("bzr",bzr);
                pramsMap.put("bzdh",bzdh);
                pramsMap.put("imageNumber",imageNumber);
                pramsMap.put("inAccountStatus",inAccountStatus);
                pramsMap.put("snVoucherNumber",snVoucherNumber);
                pramsMap.put("postingTimeEnd",postingTimeEnd);
                pramsMap.put("postingTimeStart",postingTimeStart);
                pramsMap.put("accountPeriod",accountPeriod);
                List<String> gfshList = new ArrayList<>();
                if (!StringHelper.isBlank(gfsh)) {
                    gfshList = StringHelper.stringToList(gfsh);
                    pramsMap.put("gfsh", gfshList);
                } else {
                    pramsMap.put("gfsh", gfshList);
                }
                pramsMap.put("companyCode", companyCode);
                pramsMap.put("businessCode", "99");
                if (!StringHelper.isBlank(qszt)) {
                    if (!"99".equals(qszt)) {
                        pramsMap.put("qszt", qszt);
                        if ("1".equals(qszt)) {
                            pramsMap.put("businessCode", businessCode);
                        }
                    }
                }
                if (!StringHelper.isBlank(rzzt)) {
                    if (!"99".equals(rzzt)) {
                        pramsMap.put("rzzt", rzzt);
                    }
                }
                if (!StringHelper.isBlank(yczt)) {
                    if (!"99".equals(yczt)) {
                        pramsMap.put("yczt", yczt);
                    }
                }
                pramsMap.put("kpksrq", DateUtils.getMorning(DateUtils.strToDate(kpksrq)));
                pramsMap.put("kpjsrq", DateUtils.getNight(DateUtils.strToDate(kpjsrq)));
                pramsMap.put("invoiceNo", invoiceNo);
                pramsMap.put("xfmc", xfmc);
                pramsMap.put("company", company);
                // 导出文件名
                StringBuilder ftpFileName = new StringBuilder();
                ftpFileName.append(userid);
                ftpFileName.append(fileSplit);
                ftpFileName.append(fileNamePrefix);
                ftpFileName.append(fileSplit);
                ftpFileName.append(DateUtils.getStringDateShort());
                ftpFileName.append(fileSplit);
                ftpFileName.append((new Date()).getTime());
                ftpFileName.append(fileNameSurfix);

                // 本地文件存储路径
                StringBuilder excelFile = new StringBuilder();
                excelFile.append(fileService.getLocalPathDefault());
                excelFile.append(ftpFileName.toString());

                // 递归导出数据到excel
                this.cicleExport(pramsMap, default_curr, excelFile.toString(), jsonObject);

                // FTP文件存储路径
                String[] toDay = DateUtil.today().split("-");
                String ftpFilePath = fileService.getPathPrefix() + userName + "/" + "excelFile/" + toDay[0] + "/"
                    + toDay[1] + "/" + toDay[2] + "/" + ftpFileName.toString();
                // 文件上传ftp
                fileService.uploadFile(ftpFilePath, excelFile.toString());

                // 设置成功消息内容
                msg.put("title", super.getSuccTitle(title));
                msg.put("content", super.getSuccContent(createDate));
                msg.put("url", super.getUrl(id));
                msg.put("useraccount", userName);

                // 记录导出完成状态
                exportLogService.updateSucc(id, ftpFilePath, dbName);

            } catch (Exception e) {
                String errmsg = ExceptionUtil.getExceptionDeteil(e, 4000);
                // 设置失败消息内容
                msg.put("title", super.getFailTitle(title));
                msg.put("content", super.getFailContent(createDate, errmsg));
                msg.put("url", "");
                msg.put("useraccount", userName);

                log.error("", e);
                // 记录导出出错状态
                exportLogService.updateFail(id, errmsg, dbName);
            } finally {
                // 发送导出消息到mq
                String responseResult = JsonResponseUtils.responseResult(msg);
                sender.sendToMsg(responseResult);

                // 申请成功后像websocket发送申请信息
                feginWebsocket.setMessageToWebsocket(userName);
            }
        }
    }

    /**
     * 递归导出数据到excel文件，防止jvm oom
     *
     * @param pramsMap
     *            查询数据参数集合
     * @param curr
     *            当前第几页
     * @param excelFile
     *            导出的文件
     * @throws Exception
     *
     */

    private boolean cicleExport(Map<String, Object> pramsMap, int curr, String excelFile, JSONObject jsonObject)
        throws Exception {
        // 查询指定页数的数据
        JSONObject pageData = ycfpcxService.selectByPramsMap(pramsMap, curr, default_size);
        boolean hasNext = pageData.getBooleanValue("hasNext");
        List<BbycfpcxRespVO> dataList = (List<BbycfpcxRespVO>)pageData.get("datalist");
        boolean firstPage = false;
        if (curr == default_curr) {
            firstPage = true;
        }

        // 数据不是最后一页
        if (hasNext) {
            int nextPage = pageData.getIntValue("nextPage");
            // 写入当前页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, false);
            // 更新页数
            return cicleExport(pramsMap, nextPage, excelFile, jsonObject);
        } else {
            // 写入最后一页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, true);
            return hasNext;
        }

    }
}
