package com.dxhy.erp.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dxhy.common.constant.DbConstant;
import com.dxhy.common.datasource.config.DynamicContextHolder;
import com.dxhy.common.enums.FplxEnum;
import com.dxhy.common.utils.DateUtils;
import com.dxhy.common.utils.R;
import com.dxhy.erp.dao.*;
import com.dxhy.erp.entity.sdny.*;
import com.dxhy.erp.enums.TaxRateCodeEnum;
import com.dxhy.erp.service.ImageSubmitService;
import com.dxhy.erp.service.SNPushCheckRecordService;
import com.dxhy.erp.service.SdnyTDxRecordInvoiceDetailSalesService;
import com.dxhy.erp.utils.Constant;
import com.dxhy.erp.utils.JsonUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
//@Transactional(rollbackFor = Exception.class)
@Slf4j
public class ImageSubmitServiceImpl implements ImageSubmitService {

    @Autowired
    private ImageSubmitDao imageSubmitDao;

    @Autowired
    private SdnyTDxRecordInvoiceDao tdxRecordInvoiceDao;

    @Autowired
    private SdnyTDxRecordInvoiceDetailDao detailDao;


    @Autowired
    private SdnyTDxSaleRecordInvoiceDao sdnyTDxSaleRecordInvoiceDao;

    @Autowired
    private SdnyTDxRecordInvoiceDetailSalesService sdnyTDxRecordInvoiceDetailSalesService;

    @Autowired
    private SdnyTDxRecordInvoiceDetailSalesDao detailSalesDao;

    @Autowired
    private SNPushCheckRecordService snPushCheckRecordService;

    @Override
    public R salesInvoiceSubmit(SNRequestObject params) {
        List<String> uuidList = new ArrayList<>();
        Map<String, SNInvoice> paramsInvoiceMap = new HashMap<>();
        params.getInvoiceInfoList().stream().forEach(item -> {
            String uuid = (StringUtils.isBlank(item.getInvoiceCode()) ? "" : item.getInvoiceCode()) + item.getInvoiceNumber();
            uuidList.add(uuid);
            paramsInvoiceMap.put(uuid, item);
        });
        QueryWrapper<SdnyTDxRecordInvoice> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("uuid", uuidList);
        List<SdnyTDxRecordInvoice> invoicesList = tdxRecordInvoiceDao.selectList(queryWrapper);
        if (invoicesList == null || invoicesList.size() == 0) {
            return R.error(500, "未查询到发票信息，请重新查验");
        }
        if (invoicesList.size() != uuidList.size()) {
            StringBuffer msg = new StringBuffer();
            Map<String, SdnyTDxRecordInvoice> invoiceMap = invoicesList.stream().collect(Collectors.toMap(SdnyTDxRecordInvoice::getUuid, v -> v));
            uuidList.stream().forEach(uuid -> {
                if (!invoiceMap.containsKey(uuid)) {
                    SNInvoice invoice = paramsInvoiceMap.get(uuid);
                    String errorMsg = "发票代码:" + invoice.getInvoiceCode() + ",发票号码:" + invoice.getInvoiceNumber() + ";";
                    msg.append(errorMsg);
                }
            });
            String errorMsg = msg.toString();
            return R.error(500, "未查询到如下发票，请重新查验，" + errorMsg);
        }
        QueryWrapper<SdnyTDxRecordInvoiceDetail> detailQueryWrapper = new QueryWrapper<>();
        detailQueryWrapper.in("uuid", uuidList);
        List<SdnyTDxRecordInvoiceDetail> detailList = detailDao.selectList(detailQueryWrapper);
        if (detailList == null || detailList.size() == 0) {
            return R.error(500,"未查询到发票明细信息，请重新查验");
        }
        Map<String, List<SdnyTDxRecordInvoiceDetail>> detailiMap = new HashMap<>();
        detailList.stream().forEach(item -> {
            List<SdnyTDxRecordInvoiceDetail> list = null;
            if (detailiMap.containsKey(item.getUuid())) {
                list = detailiMap.get(item.getUuid());
                list.add(item);
                detailiMap.put(item.getUuid(), list);
            } else {
                list = new ArrayList<>();
                list.add(item);
                detailiMap.put(item.getUuid(), list);
            }
        });
        StringBuffer detailErrorMsg = new StringBuffer();
        Boolean flag = false;
        for (String uuid : uuidList) {
            if (!detailiMap.containsKey(uuid)) {
                flag = true;
                SNInvoice invoice = paramsInvoiceMap.get(uuid);
                String errorMsg = "发票代码:" + invoice.getInvoiceCode() + ",发票号码:" + invoice.getInvoiceNumber() + ";";
                detailErrorMsg.append(errorMsg);
            }
        }
        if (flag) {
            String errorMsg = detailErrorMsg.toString();
            return R.error(500, "未查询到如下发票的明细行，请重新查验，" + errorMsg);
        }
        invoicesList.stream().forEach(item -> {
            item.setImageId(params.getImageId());
            item.setBzdh(params.getBillNum());
            item.setBzr(params.getJbrUserName());
            item.setCompCode(params.getCompCode());
        });
        List<SdnyTDxSaleRecordInvoice> salesInvoiceList = new ArrayList<>();
        invoicesList.stream().forEach(invoice -> {


            List<SdnyTDxSaleRecordInvoice> saleRecordInvoices = sdnyTDxSaleRecordInvoiceDao.selectList(new QueryWrapper<SdnyTDxSaleRecordInvoice>().eq("uuid", invoice.getUuid()));
            SdnyTDxSaleRecordInvoice saleRecordInvoice = null;
            if (saleRecordInvoices!=null && saleRecordInvoices.size()>0) {
                saleRecordInvoice = saleRecordInvoices.get(0);
            }
            List<SdnyTDxRecordInvoiceDetailSales> salesDetailList = new ArrayList<>();
            detailiMap.get(invoice.getUuid()).stream().forEach(item -> {
                log.info("进项表明细行：{}",JSONObject.toJSONString(item));
                SdnyTDxRecordInvoiceDetailSales salesDetail = JsonUtils.getInstance().parseObject(JSONObject.toJSONString(item), SdnyTDxRecordInvoiceDetailSales.class);
                log.info("销项表明细行：{}",JSONObject.toJSONString(salesDetail));
                salesDetail.setId(null);
                salesDetail.setTaxRate(item.getTaxRate());
                salesDetailList.add(salesDetail);
            });
            if (saleRecordInvoice == null) {
                // 新增
                saleRecordInvoice = JsonUtils.getInstance().parseObject(JSONObject.toJSONString(invoice), SdnyTDxSaleRecordInvoice.class);
                saleRecordInvoice.setId(null);
                sdnyTDxSaleRecordInvoiceDao.insert(saleRecordInvoice);

            } else {
                // 修改
                saleRecordInvoice.setImageId(invoice.getImageId());
                saleRecordInvoice.setBzdh(invoice.getBzdh());
                saleRecordInvoice.setBzr(invoice.getBzr());
                saleRecordInvoice.setCompCode(invoice.getCompCode());
                sdnyTDxSaleRecordInvoiceDao.updateById(saleRecordInvoice);
            }
            detailSalesDao.deleteByUUid(invoice.getUuid());
            detailSalesDao.batchInsert(salesDetailList);

            saleRecordInvoice.setDetailList(salesDetailList);
            saleRecordInvoice.setSort(paramsInvoiceMap.get(invoice.getUuid()).getSort());
            saleRecordInvoice.setFileId(paramsInvoiceMap.get(invoice.getUuid()).getFileId());
            saleRecordInvoice.setFileName(paramsInvoiceMap.get(invoice.getUuid()).getFileName());
            salesInvoiceList.add(saleRecordInvoice);
        });

        return pushSap(salesInvoiceList, params);
    }

    @Override
    public R salesInvoiceRollback(SNRequestObject params) {
        List<SdnyTDxSaleRecordInvoice> data = sdnyTDxSaleRecordInvoiceDao.selectList(new QueryWrapper<SdnyTDxSaleRecordInvoice>().eq("image_id", params.getImageId()));
        if (data != null && data.size() > 0) {
            List<String> uuidList = new ArrayList<>();
            data.stream().forEach(item -> {
                uuidList.add(item.getUuid());
            });
            UpdateWrapper<SdnyTDxSaleRecordInvoice> updateWrapper = new UpdateWrapper<>();
            updateWrapper.set(true, "image_id", null);
            updateWrapper.set(true, "bzdh", null);
            updateWrapper.set(true, "bzr", null);
            updateWrapper.in("uuid", uuidList);
            sdnyTDxSaleRecordInvoiceDao.update(null,updateWrapper);
        }
        // 影像调用提交前会先调用撤回 不管发票有没有这个影像ID都调一遍SAP
        return pushSap(null, params);
    }

    /**
     * 推送SAP
     *
     * @param invoiceList
     * @param flag        true提交 false撤回
     * @return
     */
    public R pushSap(List<SdnyTDxSaleRecordInvoice> invoiceList, SNRequestObject params) {
        log.info("销项发票推送SAP准备数据：{},影像参数：{}", invoiceList, params);
        String dt = DateUtil.now();
        List<ZData> zDataList = new ArrayList<>();
        if (invoiceList != null) {
            // 影像提交
            invoiceList.stream().forEach(item -> {
                String invoiceType = FplxEnum.getSapFplxdm(item.getInvoiceType());
                if (StringUtils.isBlank(invoiceType)) {
                    throw new RuntimeException("推送SAP失败，推导发票类型错误，发票代码:"+item.getInvoiceCode()+",发票代码:"+item.getInvoiceNo());
                }

                ZData zdata = new ZData();
                zdata.setZFPDM(StringUtils.isBlank(item.getInvoiceCode())?"":item.getInvoiceCode());
                zdata.setZFPHM(item.getInvoiceNo());
                zdata.setZFPLX(invoiceType);
                zdata.setZXFBM(item.getXfTaxNo());
                zdata.setZGFBH(item.getGfTaxNo());
                zdata.setZHSJE(item.getTaxAmount().toPlainString());
                zdata.setZBHSJE_DIF("0");
                zdata.setZZSE_DIF("0");
                zdata.setZHSJE_DIF("0");
                zdata.setZZBZ(item.getRemark());
                zdata.setZFPZT("0".equals(item.getInvoiceStatus())?"0":"1");
                zdata.setZKPRQ(DateUtils.dateToStr(item.getInvoiceDate(),DateUtils.YYYYMMDD));
                zdata.setMANDT(params.getClientNum());
                zdata.setZGFMC(item.getGfName());
                zdata.setZGFSH(item.getGfTaxNo());
                zdata.setZXFMC(item.getXfName());
                zdata.setZXFSH(item.getXfTaxNo());
                zdata.setZTID(StringUtils.isBlank(item.getInvoiceCode())?"":item.getInvoiceCode()+item.getInvoiceNo()+invoiceType);
                zdata.setZJYM(item.getCheckCode());
                zdata.setZGFZH(item.getGfBankAndNo());
                zdata.setZGFDH(item.getGfAddressAndPhone());
                zdata.setZXFDH(item.getXfAddressAndPhone());
                zdata.setZXFZH(item.getXfBankAndNo());
                zdata.setZPJLY("1");
                zdata.setZYGC(params.getJbrUserNum());
                zdata.setZYHM(params.getJbrUserName());
                zdata.setZQM("opentext");
                zdata.setZGSDM(params.getCompCode());
                if(!StringUtils.isBlank(params.getBillNum())){
                    zdata.setZYWDJ(params.getBillNum());
                    zdata.setZYWZT("1");
                    zdata.setZBZDH(params.getBillNum());
                }else{
                    zdata.setZYWZT("0");
                }
                zdata.setZPJZT("2");
                zdata.setZCJSJ(dt);
                zdata.setZCJR(params.getJbrUserNum());
                zdata.setZGXSJ(dt);
                zdata.setZGXR(params.getJbrUserNum());
                zdata.setZCYZT("1");
                zdata.setZCYJG("查验成功");
                zdata.setZCYRQ(dt);
                zdata.setZBHSJE_Y(item.getInvoiceAmount().toPlainString());
                zdata.setZZSE_Y(item.getTaxAmount().toPlainString());
                zdata.setZHSJE_Y(item.getTotalAmount().toPlainString());
                zdata.setZBHSJE(item.getInvoiceAmount().toPlainString());
                zdata.setFPYXBH(item.getFileId());
                zdata.setIMG_ID(params.getImageId());
                zdata.setSORT(String.valueOf(item.getSort()));
                // 增加文件名
                zdata.setZWJMC(item.getFileName());

                // 处理明细行
                List<ZData> zDatas=new ArrayList<>();
                for (SdnyTDxRecordInvoiceDetailSales detail : item.getDetailList()) {
                    detail.setTaxRateOfSap(getTaxRate(detail.getTaxRate()));
                    detail.setTaxRate(detail.getTaxRateOfSap().multiply(new BigDecimal(100)).setScale(4,BigDecimal.ROUND_HALF_UP).stripTrailingZeros().toPlainString());
                    log.info("明细行数据：{}",JSONObject.toJSONString(detail));
                    BigDecimal totalAmount = new BigDecimal(detail.getDetailAmount()).add(new BigDecimal(detail.getTaxAmount()));
                    zDatas.add(ZData.builder()
                            .ZBHSJE(detail.getDetailAmount())
                            .ZBHSJE_DIF("0")
                            .ZBHSJE_Y(detail.getDetailAmount())
                            .ZFPDM(item.getInvoiceCode())
                            .ZFPHM(item.getInvoiceNo())
                            .ZFPHH(detail.getDetailNo())
                            .ZGGXH(Optional.ofNullable(detail.getModel()).orElse("").replaceAll("\\.{3}|/|<|>", ""))  //去掉包含...和/、<、>的特殊字符
                            .ZHSJE(totalAmount.toPlainString())
                            .ZHSJE_Y(totalAmount.toPlainString())
                            .ZZSE_Y(detail.getTaxAmount())
                            .ZSLDW(detail.getUnit())
                            .ZTAX(detail.getTaxRate())
                            .ZZDJ(detail.getUnitPrice())
                            .ZZDJFP(detail.getUnitPrice())
                            .ZZSL(detail.getNum())
                            .MANDT(params.getClientNum())
                            .INVOICEDATE(zdata.getZKPRQ())
                            .INVOICETYPE(invoiceType)
                            .ZHWMC(detail.getGoodsName())
                            .ZTID(zdata.getZTID())
                            .ZSM(TaxRateCodeEnum.getCode(detail.getTaxRate()))
                            .build());
                }
                String taxRate = "";
                if (item.getDetailList().size() > 0) {
                    taxRate = item.getDetailList().stream().max(Comparator.comparing(SdnyTDxRecordInvoiceDetailSales::getTaxRateOfSap)).get().getTaxRate();
                }
                zdata.setIT_ITEM(zDatas);

                //税率
                zdata.setZTAX(taxRate);
                //税码
                zdata.setZSM(TaxRateCodeEnum.getCode(taxRate));
                zDataList.add(zdata);
            });
        }

        S4Request request = S4Request.builder()
                .SYSID("FPXT")
                .IFYWID("FI889")
                .BSKEY(params.getBillNum())
                .ZORG(params.getCompCode())
                .ZFILED4(params.getImageId())
                .ZDATA(JSONObject.toJSONString(zDataList))
                .ZFILED5(params.getClientNum()).build();
        JSONObject param = new JSONObject();
        param.put("IS_INPUT", request);
        //推送到SAP
        log.info("推送山能数据:{}", JSONObject.toJSONString(param));
        String result = snPushCheckRecordService.pushCheckResultToSAP(param.toJSONString());
        JSONObject rs = JSONObject.parseObject(result).getJSONObject("ES_OUTPUT");
        if (!"S".equals(rs.getString("ZTYPE"))) {
            return R.error(500,rs.getString("ZMESSAGE"));
        }
        return R.success();
    }

    /**
     * 处理税率
     * @param taxRate
     * @return 去掉%的小数
     */
    public static BigDecimal getTaxRate(String taxRate) {
        if (StringUtils.isBlank(taxRate) || Constant.ZERO_TAXRATE_VALUE.contains(taxRate)) {
            taxRate = "0";
        } else {
            taxRate = taxRate.replace("%", "");
            // 判断是不是数字 不是数字的就给0
            if (!isNumeric(taxRate)) {
                taxRate = "0";
            } else {
                // 不知道库里存的是13% 还是0.13 所以加一个判断
                if (new BigDecimal(taxRate).compareTo(BigDecimal.ONE) >= 0) {
                    // 存的是整数 13/1/1.5等
                    taxRate = new BigDecimal(taxRate).divide(new BigDecimal(100)).toPlainString();
                } else {
                    // 存的是小数 0.13 0.015等就不处理了 直接返回
                }
            }
        }
        return new BigDecimal(taxRate);
    }

    public static boolean isNumeric(String str){
        try {
            Double.parseDouble(str);
            return true;
        } catch (NumberFormatException e) {
            return false;
        }

    }

    public static void main(String[] args) {
        BigDecimal a = new BigDecimal(0.13);
        String tax = a.multiply(new BigDecimal(100)).setScale(4,BigDecimal.ROUND_HALF_UP).stripTrailingZeros().toPlainString();
        System.out.println(StringUtils.isNumeric("13.00"));
        System.out.println(isNumeric("1e1"));
        System.out.println(getTaxRate("13.00"));
        System.out.println(TaxRateCodeEnum.getCode(tax));
    }
}
