
package com.dxhy.domestic.service.ckzrz.impl;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

import javax.annotation.Resource;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dxhy.common.constant.DbConstant;
import com.dxhy.common.constant.DictConstant;
import com.dxhy.common.datasource.config.DynamicContextHolder;
import com.dxhy.common.service.CustomsLogService;
import com.dxhy.common.service.DictdetaServcice;
import com.dxhy.common.service.TabulateRedisService;
import com.dxhy.common.util.AmountFormatUtil;
import com.dxhy.common.utils.DateUtils;
import com.dxhy.common.vo.CustomsLog;
import com.dxhy.common.vo.UserInfo;
import com.dxhy.domestic.dao.DomesticExportCustomsDao;
import com.dxhy.domestic.entity.TdxDkCount;
import com.dxhy.domestic.entity.TdxExportCustoms;
import com.dxhy.domestic.entity.TdxTaxCurrent;
import com.dxhy.domestic.model.dto.ResultData;
import com.dxhy.domestic.model.dto.TdxExportCustomsModelDTO;
import com.dxhy.domestic.service.ckzrz.DomesticDkCountService;
import com.dxhy.domestic.service.ckzrz.DomesticExportCustomsCkService;
import com.dxhy.domestic.service.ckzrz.DomesticTaxCurrentService;
import com.dxhy.domestic.util.DateUtil;
import com.dxhy.domestic.util.ObjectUtil;

import cn.hutool.core.codec.Base64;

/**
 * 业务单元
 *
 * @author dxhy
 */
@SuppressWarnings("AlibabaMethodTooLong")
@Service
@Slf4j
public class DomesticExportCustomsCkServiceImpl extends ServiceImpl<DomesticExportCustomsDao, TdxExportCustoms>
    implements DomesticExportCustomsCkService {

    @Resource
    private DomesticTaxCurrentService domesticTaxCurrentService;

    @Resource
    private DomesticDkCountService domesticDkCountService;

    @Resource
    private CustomsLogService logService;

    @Resource
    private TabulateRedisService tabulateRedisService;
    @Resource
    private DictdetaServcice dictdetaServcice;

    @Override
    public String querySgByPage(Page page, TdxExportCustomsModelDTO dto, UserInfo userInfo) throws Exception {
        dto.setParam(page, userInfo);

        ResultData<Object> resultData = new ResultData<>();
        JSONObject tabulateMenu = tabulateRedisService.getTabulateMenu(dto.getCompany(), dto.getMenuId());
        resultData.putTabulateMenu(tabulateMenu);

        Map<String, Object> mapTj = null;

        // 合计信息,仅第一页时需要统计，减少数据库压力
        if (dto.getCurr() == 1) {
            mapTj = this.baseMapper.querySgBySum(dto);
        }

        Page<Map<String, Object>> result = this.baseMapper.querySgByPage(page, dto);
        if (ResultData.isEmpty(result)) {
            // List<DictdetaModel> lyModels = dictdetaServcice.selectDictdeta("jksSources");
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            for (int i = 0; i < page.getRecords().size(); i++) {
                Map<String, Object> map = result.getRecords().get(i);

                String dataSources = (String)map.get("dataSources");

                map.put("xh",
                    String.valueOf((((page.getCurrent() == 0 ? 1 : page.getCurrent()) - 1) * page.getSize()) + i + 1));
                if (map.get("fillInDate") != null) {
                    map.put("fillInDate",map.get("fillInDate"));
                }
                map.put("dataSources", dictdetaServcice.queryDictName(DictConstant.JKSCJFS, dataSources));

                if ("1".equals(map.get("inAccountStatus"))){
                    map.put("inAccountStatus","已入账");
                }else {
                    map.put("inAccountStatus","未入账");
                }

                if (map.get("exportTaxAmount") == null || "".equals(map.get("exportTaxAmount"))) {
                    map.put("exportTaxAmount", "0.00");
                } else {
                    map.put("exportTaxAmount",
                        AmountFormatUtil.fmtMicrometer(new BigDecimal(ObjectUtil.fixNull(map.get("exportTaxAmount")))
                            .setScale(2, BigDecimal.ROUND_HALF_UP).toPlainString()));
                }
                if (map.get("yxExportTaxAmount") == null || "".equals(map.get("yxExportTaxAmount"))) {
                    map.put("yxExportTaxAmount", "0.00");
                } else {
                    map.put("yxExportTaxAmount", new BigDecimal(ObjectUtil.fixNull(map.get("yxExportTaxAmount")))
                        .setScale(2, BigDecimal.ROUND_HALF_UP).toPlainString());
                }
                map.put("select", true);
                map.put("upgradeStatus", "1");

                TdxDkCount tDxDkCount;
                TdxTaxCurrent tDxTaxCurrent;
                List<TdxTaxCurrent> tdxTaxCurrents = domesticTaxCurrentService
                    .list(Wrappers.<TdxTaxCurrent>query().lambda().eq(TdxTaxCurrent::getTaxno, map.get("gfTaxNo")));

                if (tdxTaxCurrents == null || tdxTaxCurrents.size() == 0) {
                    map.put("select", false);
                    continue;
                }
                tDxTaxCurrent = tdxTaxCurrents.get(0);
                List<TdxDkCount> tdxDkCounts = domesticDkCountService
                    .list(Wrappers.<TdxDkCount>query().lambda().eq(TdxDkCount::getTaxno, map.get("gfTaxNo"))
                        .eq(TdxDkCount::getSkssq, tDxTaxCurrent.getCurrentTaxPeriod()));

                if (tdxDkCounts == null || tdxDkCounts.size() == 0) {
                    map.put("select", false);
                    continue;
                }

                tDxDkCount = tdxDkCounts.get(0);
                String fillInDate = (String)map.get("fillInDate");
                if(map.get("postingTime") != null) {
                    LocalDateTime dateTime = (LocalDateTime) map.get("postingTime");
                    String format = dateTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                    map.put("postingTime", format);
                }
                if (!DateUtil.isInTwoDate(tDxTaxCurrent.getSelectStartDate(), tDxTaxCurrent.getSelectEndDate(),
                    fillInDate)) {
                    map.put("select", false);
                    continue;
                }

                if (!"0".equals(tDxDkCount.getTjStatus()) && !"3".equals(tDxDkCount.getTjStatus())) {
                    map.put("select", false);
                    continue;
                }

                String applyCheck = (String)map.get("applyCheck");
                String checkStatus = (String)map.get("checkStatus");
                // 采集数据
                if ("0".equals(dataSources)) {
                    if (!"0".equals(map.get("glzt"))) {
                        map.put("select", false);
                    }
                } else {// 录入数据
                    if ("0".equals(applyCheck) && "1".equals(checkStatus)) {
                        log.debug("");
                    } else if ("1".equals(applyCheck) && "0".equals(checkStatus)) {
                        log.debug("");
                    } else {
                        map.put("select", false);
                    }
                }

            }
        }
        return resultData.getTjBdData(mapTj).getPageData(result).toJsonBase();
    }

    @Override
    public String approveSg(TdxExportCustomsModelDTO dto, UserInfo userInfo) {

        Map<String, Object> resultMap = new HashMap<>(16);

        long total = 0;
        BigDecimal hjje = new BigDecimal(0);
        BigDecimal hjse = new BigDecimal(0);
        BigDecimal hjyxse = new BigDecimal(0);
        String message = "";

        boolean flag = false;
        boolean flag1 = false;
        boolean flag2 = false;
        boolean flag3 = false;

        List<String> sb1List = new ArrayList<>();
        List<String> sb2List = new ArrayList<>();
        List<String> sb3List = new ArrayList<>();

        StringBuilder sb = new StringBuilder();
        StringBuilder sb1 = new StringBuilder();
        StringBuilder sb2 = new StringBuilder();
        StringBuilder sb3 = new StringBuilder();

        sb.append("以下缴款书的抵扣税额小于等于0或大于税额不可勾选认证： 缴款书号码\n ");
        sb1.append("以下税号已被锁定不可勾选认证： 税号\n");
        sb2.append("以下税号已申请抵扣统计，请撤销统计后重新操作：税号  \n ");
        sb3.append("以下税号本月缴款书可认证总税额超限：税号\n");

        // 1.查询数据
        List<TdxExportCustoms> records = new ArrayList<>();
        // 2.校验
        List<String> dkList = new ArrayList<>();
        // 抵扣限额合计
        BigDecimal dkTaxAmount = new BigDecimal(0);
        List<CustomsLog> logList = new ArrayList<>();
        for (String idstr : dto.getIds()) {

            if (idstr == null) {
                continue;
            }

            String[] arr = idstr.split(":");
            Long id = Long.parseLong(arr[0]);
            // 有效税额
            String yxTaxAmount = arr[1];

            if (yxTaxAmount != null) {
                yxTaxAmount = yxTaxAmount.replaceAll(",", "");
            }

            TdxExportCustoms record = this.baseMapper.selectById(id);
            if (record != null) {

                TdxTaxCurrent tDxTaxCurrent = null;
                TdxDkCount tDxDkCount = null;
                List<TdxTaxCurrent> tdxTaxCurrents = domesticTaxCurrentService
                    .list(Wrappers.<TdxTaxCurrent>query().lambda().eq(TdxTaxCurrent::getTaxno, record.getGfTaxNo()));

                tDxTaxCurrent = tdxTaxCurrents.get(0);

                List<TdxDkCount> tdxDkCounts = domesticDkCountService
                    .list(Wrappers.<TdxDkCount>query().lambda().eq(TdxDkCount::getTaxno, record.getGfTaxNo())
                        .eq(TdxDkCount::getSkssq, tDxTaxCurrent.getCurrentTaxPeriod()));

                tDxDkCount = tdxDkCounts.get(0);

                // 认证抵扣
                if ("1".equals(dto.getRzlx())) {
                    if (Double.parseDouble(yxTaxAmount) <= 0
                        || Double.parseDouble(yxTaxAmount) > Double.parseDouble(record.getExportTaxAmount())) {
                        sb.append(record.getCustomsCode()).append(" \n ");
                        flag = true;
                        continue;
                    }
                    if ("1".equals(tDxTaxCurrent.getLatchStatus())) {
                        if (!sb1List.contains(record.getGfTaxNo())) {
                            sb1.append(record.getGfTaxNo()).append(" \n ");
                            sb1List.add(record.getGfTaxNo());
                        }
                        flag1 = true;
                        continue;
                    }
                    if (!"0".equals(tDxDkCount.getTjStatus()) && !"3".equals(tDxDkCount.getTjStatus())) {
                        if (!sb2List.contains(record.getGfTaxNo())) {
                            sb2.append(record.getGfTaxNo()).append(" \n ");
                            sb2List.add(record.getGfTaxNo());
                        }
                        flag2 = true;
                        continue;
                    }

                    // 判断抵扣限额
                    if (tDxTaxCurrent.getLimitDkse() != null
                        && tDxTaxCurrent.getLimitDkse().compareTo(BigDecimal.ZERO) != 0) {
                        if (!dkList.contains(record.getGfTaxNo())) {

                            Map<String, Object> paramMap = new HashMap<>(16);
                            paramMap.put("gfsh", record.getGfTaxNo());
                            paramMap.put("skssq", tDxTaxCurrent.getCurrentTaxPeriod());
                            TdxExportCustoms dkInvoice = super.baseMapper.selectDkse(paramMap);

                            if (dkInvoice != null && dkInvoice.getYxExportTaxAmount() != null) {
                                dkTaxAmount = dkTaxAmount
                                    .add(BigDecimal.valueOf(Double.parseDouble(dkInvoice.getYxExportTaxAmount())));
                            }
                            dkTaxAmount = dkTaxAmount.add(BigDecimal.valueOf(Double.parseDouble(yxTaxAmount)));
                            if (dkTaxAmount.compareTo(tDxTaxCurrent.getLimitDkse()) > 0) {
                                dkList.add(record.getGfTaxNo());

                                if (!sb3List.contains(record.getGfTaxNo())) {
                                    sb3.append(record.getGfTaxNo()).append(" \n ");
                                    sb3List.add(record.getGfTaxNo());
                                }
                                flag3 = true;
                                continue;
                            }

                        }
                    }

                    record.setYxExportTaxAmount(yxTaxAmount);
                    record.setRzlx("1");
                    record.setAuthStatus("2");
                    record.setBdkStatus("1");
                    record.setConfirmDate(DateUtils.getSqlDate());
                    record.setConfirmUser(userInfo.getUsername());
                    record.setRzhBelongDate(tDxTaxCurrent.getCurrentTaxPeriod());
                    record.setCxrzStatus("0");
                    record.setCxrzDate(null);
                    record.setRzhType(dto.getRzhType());

                    records.add(record);

                    CustomsLog customsLog = new CustomsLog();
                    customsLog.setCustomsCode(record.getCustomsCode());
                    customsLog.setCreateDate(new Date());
                    customsLog.setInputName(userInfo.getLoginname());
                    customsLog.setType("8");
                    customsLog.setInputStatus("14");
                    logList.add(customsLog);
                } else if ("2".equals(dto.getRzlx())) {
                    if ("1".equals(tDxTaxCurrent.getLatchStatus())) {
                        if (!sb1List.contains(record.getGfTaxNo())) {
                            sb1.append(record.getGfTaxNo()).append(" \n ");
                            sb1List.add(record.getGfTaxNo());
                        }
                        flag1 = true;
                        continue;
                    }
                    if (!"0".equals(tDxDkCount.getTjStatus()) && !"3".equals(tDxDkCount.getTjStatus())) {
                        if (!sb2List.contains(record.getGfTaxNo())) {
                            sb2.append(record.getGfTaxNo()).append(" \n ");
                            sb2List.add(record.getGfTaxNo());
                        }
                        flag2 = true;
                        continue;
                    }

                    record.setRzlx("4");
                    record.setBdkStatus("2");
                    record.setAuthStatus("2");
                    record.setRzhBelongDate(tDxTaxCurrent.getCurrentTaxPeriod());
                    record.setConfirmDate(DateUtils.getSqlDate());
                    record.setConfirmUser(userInfo.getUsername());
                    record.setRzhType(dto.getRzhType());
                    record.setCxrzDate(null);
                    record.setCxrzStatus("0");
                    records.add(record);

                    CustomsLog customsLog = new CustomsLog();
                    customsLog.setCustomsCode(record.getCustomsCode());
                    customsLog.setCreateDate(new Date());
                    customsLog.setInputName(userInfo.getLoginname());
                    customsLog.setType("9");
                    customsLog.setInputStatus("7");
                    logList.add(customsLog);
                }

            }
        }
        // 3.更新数据

        if (!flag && !flag1 && !flag2 && !flag3) {
            for (TdxExportCustoms record : records) {

                total++;
                hjse = hjse.add(new BigDecimal(record.getExportTaxAmount()));
                if (record.getYxExportTaxAmount() != null) {
                    hjyxse = hjyxse.add(new BigDecimal(record.getYxExportTaxAmount()));
                }

                this.baseMapper.updateById(record);
            }
        }
        if (logList.size() > 0) {
            logService.toCustomsLog(logList);
        }

        if (flag) {
            message = sb.toString();
        }
        if (flag1) {
            message = message + sb1.toString() + "如需勾选认证，请到企业税务信息菜单解除锁定";
        }
        if (flag2) {
            message = message + sb2.toString();
        }
        if (flag3) {
            message = message + sb3.toString();
        }

        resultMap.put("total", total);
        resultMap.put("hjse", hjse.setScale(2, BigDecimal.ROUND_HALF_UP).toPlainString());
        resultMap.put("hjyxse", hjyxse.setScale(2, BigDecimal.ROUND_HALF_UP).toPlainString());
        resultMap.put("message", message);

        return Base64.encode(JSONObject.toJSONString(resultMap));

    }

    @Override
    public String queryBdkByPage(Page page, TdxExportCustomsModelDTO dto, UserInfo userInfo) throws Exception {
        dto.setParam(page, userInfo);

        ResultData<Object> resultData = new ResultData<>();
        JSONObject tabulateMenu = tabulateRedisService.getTabulateMenu(dto.getCompany(), dto.getMenuId());
        resultData.putTabulateMenu(tabulateMenu);

        if (StringUtils.isBlank(dto.getGfsh())) {
            resultData.setDataList(new ArrayList<>());
            return resultData.toJsonBase();
        }
        Map mapTj = null;

        // 合计信息,仅第一页时需要统计，减少数据库压力
        if (dto.getCurr() == 1) {
            mapTj = this.baseMapper.queryBdkBySum(dto);
        }

        Page<Map> result = this.baseMapper.queryBdkByPage(page, dto);
        if (ResultData.isEmpty(result)) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            for (int i = 0; i < page.getRecords().size(); i++) {
                Map<String, Object> map = result.getRecords().get(i);

                String dataSources = (String)map.get("dataSources");
                String authStatus = (String)map.get("authStatus");
                String cxbdkStatus = (String)map.get("cxbdkStatus");
                String bdkStatus = (String)map.get("bdkStatus");
                map.put("id", String.valueOf(map.get("id")));
                map.put("xh",
                    String.valueOf((((page.getCurrent() == 0 ? 1 : page.getCurrent()) - 1) * page.getSize()) + i + 1));
                if (map.get("fillInDate") != null) {
                    map.put("fillInDate", map.get("fillInDate"));
                }
                // 字典转换
                map.put("dataSources", dictdetaServcice.queryDictName(DictConstant.JKSCJFS, dataSources));
                map.put("cxbdkStatus", dictdetaServcice.queryDictName(DictConstant.BDK_UNDO_STATUS, cxbdkStatus));
                map.put("authStatus", dictdetaServcice.queryDictName(DictConstant.BDK_AUTH_STATUS, authStatus));

                if ("1".equals(map.get("inAccountStatus"))){
                    map.put("inAccountStatus","已入账");
                }else {
                    map.put("inAccountStatus","未入账");
                }

                if (map.get("exportTaxAmount") == null || "".equals(map.get("exportTaxAmount"))) {
                    map.put("exportTaxAmount", "0.00");
                } else {
                    map.put("exportTaxAmount",
                        AmountFormatUtil.fmtMicrometer(new BigDecimal(ObjectUtil.fixNull(map.get("exportTaxAmount")))
                            .setScale(2, BigDecimal.ROUND_HALF_UP).toPlainString()));
                }
                if (map.get("yxExportTaxAmount") == null || "".equals(map.get("yxExportTaxAmount"))) {
                    map.put("yxExportTaxAmount", "0.00");
                } else {
                    map.put("yxExportTaxAmount",
                        AmountFormatUtil.fmtMicrometer(new BigDecimal(ObjectUtil.fixNull(map.get("yxExportTaxAmount")))
                            .setScale(2, BigDecimal.ROUND_HALF_UP).toPlainString()));
                }
                map.put("select", true);

                TdxDkCount tDxDkCount = null;
                TdxTaxCurrent tDxTaxCurrent = null;
                List<TdxTaxCurrent> tdxTaxCurrents = domesticTaxCurrentService
                    .list(Wrappers.<TdxTaxCurrent>query().lambda().eq(TdxTaxCurrent::getTaxno, map.get("gfTaxNo")));

                if (tdxTaxCurrents == null || tdxTaxCurrents.size() == 0) {
                    map.put("select", false);
                    continue;
                }
                tDxTaxCurrent = tdxTaxCurrents.get(0);
                List<TdxDkCount> tdxDkCounts = domesticDkCountService
                    .list(Wrappers.<TdxDkCount>query().lambda().eq(TdxDkCount::getTaxno, map.get("gfTaxNo"))
                        .eq(TdxDkCount::getSkssq, tDxTaxCurrent.getCurrentTaxPeriod()));

                if (tdxDkCounts == null || tdxDkCounts.size() == 0) {
                    map.put("select", false);
                    continue;
                }

                tDxDkCount = tdxDkCounts.get(0);

                if ("0".equals(dto.getSfDkStatus())) {
                    // 认证不抵扣
                    if (!"0".equals(authStatus) && !"4".equals(authStatus) && !"5".equals(authStatus)
                        && !"6".equals(authStatus)) {
                        map.put("select", false);
                        continue;
                    }
                } else if ("4".equals(dto.getSfDkStatus())) {
                    // 页面中有撤销不抵扣按钮
                    if (!"0".equals(cxbdkStatus) && !"4".equals(cxbdkStatus) && !"5".equals(cxbdkStatus)) {
                        map.put("select", false);
                        continue;
                    }

                    if (!"2".equals(bdkStatus)) {
                        map.put("select", false);
                        continue;
                    }
                }

                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
                String fillInDate = (String)map.get("fillInDate");
                if (!DateUtil.isInTwoDate(tDxTaxCurrent.getSelectStartDate(), tDxTaxCurrent.getSelectEndDate(),
                    fillInDate)) {
                    map.put("select", false);
                    continue;
                }

                if (!"0".equals(tDxDkCount.getTjStatus()) && !"3".equals(tDxDkCount.getTjStatus())) {
                    map.put("select", false);
                }

            }
        }
        return resultData.getTjBdData(mapTj).getPageData(result).toJsonBase();
    }

    @Override
    public String queryRzcxByPage(Page page, TdxExportCustomsModelDTO dto, UserInfo userInfo) throws Exception {
        dto.setParam(page, userInfo);

        ResultData<Object> resultData = new ResultData<>();
        JSONObject tabulateMenu = tabulateRedisService.getTabulateMenu(dto.getCompany(), dto.getMenuId());
        resultData.putTabulateMenu(tabulateMenu);

        if (StringUtils.isBlank(dto.getGfsh())) {
            resultData.setDataList(new ArrayList<>());
            return resultData.toJsonBase();
        }

        Map mapTj = null;

        // 合计信息,仅第一页时需要统计，减少数据库压力
        if (dto.getCurr() == 1) {
            mapTj = this.baseMapper.queryRzcxBySum(dto);
        }

        Page<Map> result = this.baseMapper.queryRzcxByPage(page, dto);
        if (ResultData.isEmpty(result)) {
            for (int i = 0; i < page.getRecords().size(); i++) {
                Map<String, Object> map = result.getRecords().get(i);

                String cxrzStatus = (String)map.get("cxrzStatus");
                String bdkStatus = (String)map.get("bdkStatus");
                String rzhYesorno = (String)map.get("rzhYesorno");

                map.put("xh",
                    String.valueOf((((page.getCurrent() == 0 ? 1 : page.getCurrent()) - 1) * page.getSize()) + i + 1));
                if (map.get("fillInDate") != null) {
                    map.put("fillInDate", map.get("fillInDate"));
                }
                if (map.get("rzhDate") != null) {
                    map.put("rzhDate", map.get("rzhDate"));
                }
                if(map.get("postingTime") != null) {
                    LocalDateTime dateTime = (LocalDateTime) map.get("postingTime");
                    String format = dateTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                    map.put("postingTime", format);
                }

                // 字典值转换
                String dataSources = (String)map.get("dataSources");
                map.put("dataSources", dictdetaServcice.queryDictName(DictConstant.JKSCJFS, dataSources));
                String glzt = (String)map.get("glzt");
                map.put("glzt", dictdetaServcice.queryDictName(DictConstant.GLZT, glzt));
                map.put("rzhYesorno", dictdetaServcice.queryDictName(DictConstant.DEDU_STATUS, rzhYesorno));
                String authStatus = (String)map.get("authStatus");
                map.put("authStatus", dictdetaServcice.queryDictName(DictConstant.AUTH_STATUS_ALL, authStatus));
                map.put("cxrzStatus", dictdetaServcice.queryDictName(DictConstant.CXRZ_STATUS, cxrzStatus));;
                map.put("bdkStatus", dictdetaServcice.queryDictName(DictConstant.BDK_STATUS, bdkStatus));
                if ("1".equals(map.get("inAccountStatus"))){
                    map.put("inAccountStatus","已入账");
                }else {
                    map.put("inAccountStatus","未入账");
                }
                map.put("bdkStatusBak", bdkStatus);
                if (map.get("exportTaxAmount") == null || "".equals(map.get("exportTaxAmount"))) {
                    map.put("exportTaxAmount", "0.00");
                } else {
                    map.put("exportTaxAmount",
                        AmountFormatUtil.fmtMicrometer(new BigDecimal(ObjectUtil.fixNull(map.get("exportTaxAmount")))
                            .setScale(2, BigDecimal.ROUND_HALF_UP).toPlainString()));
                }
                if (map.get("yxExportTaxAmount") == null || "".equals(map.get("yxExportTaxAmount"))) {
                    map.put("yxExportTaxAmount", "0.00");
                } else {
                    map.put("yxExportTaxAmount",
                        AmountFormatUtil.fmtMicrometer(new BigDecimal(ObjectUtil.fixNull(map.get("yxExportTaxAmount")))
                            .setScale(2, BigDecimal.ROUND_HALF_UP).toPlainString()));
                }
                map.put("select", true);

                if (!"1".equals(rzhYesorno)) {
                    map.put("select", false);
                    continue;
                }

                TdxDkCount tDxDkCount = null;
                TdxTaxCurrent tDxTaxCurrent = null;
                List<TdxTaxCurrent> tdxTaxCurrents = domesticTaxCurrentService
                    .list(Wrappers.<TdxTaxCurrent>query().lambda().eq(TdxTaxCurrent::getTaxno, map.get("gfTaxNo")));

                if (tdxTaxCurrents == null || tdxTaxCurrents.size() == 0) {
                    map.put("select", false);
                    continue;
                }
                tDxTaxCurrent = tdxTaxCurrents.get(0);
                List<TdxDkCount> tdxDkCounts = domesticDkCountService
                    .list(Wrappers.<TdxDkCount>query().lambda().eq(TdxDkCount::getTaxno, map.get("gfTaxNo"))
                        .eq(TdxDkCount::getSkssq, tDxTaxCurrent.getCurrentTaxPeriod()));

                if (tdxDkCounts == null || tdxDkCounts.size() == 0) {
                    map.put("select", false);
                    continue;
                }

                tDxDkCount = tdxDkCounts.get(0);

                if (map.get("rzhBelongDate") != null
                    && !map.get("rzhBelongDate").equals(tDxTaxCurrent.getCurrentTaxPeriod())) {
                    map.put("select", false);
                    continue;
                }

                if (!"0".equals(tDxDkCount.getTjStatus()) && !"3".equals(tDxDkCount.getTjStatus())) {
                    map.put("select", false);
                    continue;
                }

                if (!"0".equals(cxrzStatus) && !"5".equals(cxrzStatus)) {
                    map.put("select", false);
                }
            }
        }
        return resultData.getTjBdData(mapTj).getPageData(result).toJsonBase();
    }

    @Override
    public String approveCxrz(TdxExportCustomsModelDTO dto, UserInfo userInfo) {

        Map<String, Object> resultMap = new HashMap<>(16);

        long total = 0;
        BigDecimal hjje = new BigDecimal(0);
        BigDecimal hjse = new BigDecimal(0);
        BigDecimal hjyxse = new BigDecimal(0);
        String message = "";

        boolean flag1 = false;
        boolean flag2 = false;
        boolean flag3 = false;

        List<String> sb1List = new ArrayList<>();
        List<String> sb2List = new ArrayList<>();

        StringBuilder sb1 = new StringBuilder();
        StringBuilder sb2 = new StringBuilder();
        StringBuilder sb3 = new StringBuilder();

        sb1.append("以下税号已被锁定不可勾选认证：税号 \n");
        sb2.append("以下税号已申请抵扣统计，请撤销统计后重新操作：税号\n ");
        sb3.append("以下缴款书的税款所属期不是当前属期：缴款书号\n ");

        // 1.查询数据
        List<TdxExportCustoms> records = new ArrayList<>();
        // 2.校验
        List<String> dkList = new ArrayList<>();
        List<CustomsLog> logList = new ArrayList<>();
        for (String idstr : dto.getIds()) {

            Long id = Long.parseLong(idstr);

            TdxExportCustoms record = this.baseMapper.selectById(id);
            if (record != null) {

                TdxTaxCurrent tDxTaxCurrent = null;
                TdxDkCount tDxDkCount = null;
                List<TdxTaxCurrent> tdxTaxCurrents = domesticTaxCurrentService
                    .list(Wrappers.<TdxTaxCurrent>query().lambda().eq(TdxTaxCurrent::getTaxno, record.getGfTaxNo()));

                tDxTaxCurrent = tdxTaxCurrents.get(0);

                List<TdxDkCount> tdxDkCounts = domesticDkCountService
                    .list(Wrappers.<TdxDkCount>query().lambda().eq(TdxDkCount::getTaxno, record.getGfTaxNo())
                        .eq(TdxDkCount::getSkssq, tDxTaxCurrent.getCurrentTaxPeriod()));

                tDxDkCount = tdxDkCounts.get(0);

                if ("1".equals(tDxTaxCurrent.getLatchStatus())) {
                    if (!sb1List.contains(record.getGfTaxNo())) {
                        sb1.append(record.getGfTaxNo()).append(" \n ");
                        sb1List.add(record.getGfTaxNo());
                    }
                    flag1 = true;
                    continue;
                }

                if (!"0".equals(tDxDkCount.getTjStatus()) && !"3".equals(tDxDkCount.getTjStatus())) {
                    if (!sb2List.contains(record.getGfTaxNo())) {
                        sb2.append(record.getGfTaxNo()).append(" \n ");
                        sb2List.add(record.getGfTaxNo());
                    }
                    flag2 = true;
                    continue;
                }

                // 撤销抵扣
                if ("1".equals(record.getBdkStatus())) {
                    record.setRzlx("5");
                    record.setCxrzCurrentPeriod(tDxTaxCurrent.getCurrentTaxPeriod());
                    record.setCxrzDate(new Date());
                    record.setCxrzStatus("1");
                    records.add(record);

                    CustomsLog customsLog = new CustomsLog();
                    customsLog.setCustomsCode(record.getCustomsCode());
                    customsLog.setCreateDate(new Date());
                    customsLog.setType("8");
                    customsLog.setInputName(userInfo.getLoginname());
                    customsLog.setInputStatus("15");
                    logList.add(customsLog);

                } else if ("2".equals(record.getBdkStatus())) {
                    // 撤销不抵扣
                    record.setBdkCurrentPeriod(tDxTaxCurrent.getCurrentTaxPeriod());
                    record.setRzlx("6");
                    record.setCxbdkDate(new Date());
                    record.setCxbdkStatus("1");
                    records.add(record);

                    CustomsLog customsLog = new CustomsLog();
                    customsLog.setCustomsCode(record.getCustomsCode());
                    customsLog.setCreateDate(new Date());
                    customsLog.setType("9");
                    customsLog.setInputName(userInfo.getLoginname());
                    customsLog.setInputStatus("10");
                    logList.add(customsLog);
                }
            }
        }
        // 3.更新数据
        for (TdxExportCustoms record : records) {

            total++;
            hjse = hjse.add(new BigDecimal(record.getExportTaxAmount()));
            if (record.getYxExportTaxAmount() != null) {
                hjyxse = hjyxse.add(new BigDecimal(record.getYxExportTaxAmount()));
            }
            this.baseMapper.updateById(record);
        }

        if (logList.size() > 0) {
            logService.toCustomsLog(logList);
        }

        if (flag1) {
            message = message + sb1.toString() + "如需勾选认证，请到企业税务信息菜单解除锁定";
        }
        if (flag2) {
            message = message + sb2.toString();
        }

        if (flag3) {
            message = message + sb3.toString();
        }

        resultMap.put("total", total);
        resultMap.put("hjse", hjse);
        resultMap.put("message", message);
        resultMap.put("hjje", hjje);
        resultMap.put("hjdk", hjyxse);
        return Base64.encode(JSONObject.toJSONString(resultMap));
    }
}
